*! predict_ldm version 0.25 BETA Paul Allison & Richard Williams July 29, 2020
****************************************************************
/* The following program is intended for use after running the
reg command with a binary dependent variable. It produces yhat,
a variable containing the usual predicted values from a linear 
regression, and yhat_ldm, a variable containing predicted 
probabilities that are bounded by (0,1). These are based on a linear
discriminant model. OLS coefficients are "corrected" to become logistic
coefficients, which are then used to generate predictions based on the
logistic model. See G.W. Haggstrom (1983) for details.  
*****************************************************************/

program define predict_ldm
	if "`e(cmd)'" != "regress"  {
	    display as error "You must first run regress with a binary dependent variable"
		exit
	}
	local depvar = e(depvar)
	capture assert `depvar' == 0 | `depvar' == 1 if e(sample)
	if _rc != 0 {
	    display as error "You must first run regress with a binary dependent variable"
		exit
	}
	tempname nomiss ybar k_ b_ cons_ a_ logodds
	
	syntax [newvarname] [if] [in]
	if "`varlist'" == "" {
		local yhat yhat
	}
	else {
		local yhat `varlist'
	}
	marksample touse, novarlist
	quietly predict `yhat' if `touse'

	
	
	gen `nomiss'=e(sample)
	summarize `depvar' if `nomiss'==1, meanonly
	scalar `ybar'=r(mean)
	scalar `k_' =e(N)/e(rss)
	matrix `b_'=e(b)
	scalar `cons_'=`b_'[1,e(rank)]
	scalar `a_' = log(`ybar'/(1-`ybar')) + `k_'*(`cons_'-.5) ///
		+ .5*(1/`ybar' - 1/(1-`ybar'))
	gen `logodds'=`k_'*(`yhat'-`cons_')+ `a_' if `touse'
	gen `yhat'_ldm = 1/(1+exp(-`logodds')) if `touse'
	label variable `yhat'_ldm "Pr(`depvar')"
end
